@@ -72,8 +72,6 @@ public:
     virtual void stop () {}
     /*virtual*/ void dismiss ();
 
-    Mrl *mrl ();
-
 protected:
     Actor (ActorAgent *manager, Node *node);
     virtual ~Actor ();
@@ -123,7 +121,8 @@ private:
  * Actor for audio/video, groups Mrl, Process and Viewer
  */
 
-class KMPLAYER_NO_EXPORT AudioVideoActor : public Actor {
+class KMPLAYER_NO_EXPORT AudioVideoActor : public Actor, public MrlBackEndNotify
+{
     friend class ActorAgent;
 public:
     //enum Request { ask_nothing, ask_play, ask_stop, ask_delete };
@@ -137,6 +136,11 @@ public:
     virtual void pause ();
     virtual void unpause ();
 
+    virtual Mrl *sourceMrl ();
+    virtual void referenceMrl (const String &url);
+    virtual void infoMessage (const String &msg);
+    virtual void readyToPlay (Process *process);
+
 protected:
     ~AudioVideoActor ();
 };
