@@ -21,33 +21,56 @@
  */
 
 #include <stdio.h>
+#include <unistd.h>
 
 #include "log.h"
 
 using namespace KMPlayer;
 
 
+static FILE *log_out;
+
+static FILE *getLog () {
+    if (!log_out) {
+        log_out = isatty (2) ? stderr : fopen ("/home/user/.kmplayer/log.txt", "w+");
+        FILE *f = fopen ("/home/user/.kmplayer/err.txt", "w+");
+        fprintf(f, "isaaty %d out %p stderr %p\n", isatty(2), log_out, stderr);
+        fclose(f);
+    }
+    return log_out;
+}
+
 KMPLAYER_NO_EXPORT Log & Log::operator << (const char * s) {
-    fprintf (stderr, "%s", s ? s : "<NULL>");
+    FILE *out = getLog ();
+    if (out)
+        fprintf (out, "%s", s ? s : "<NULL>");
     return *this;
 }
 
 KMPLAYER_NO_EXPORT Log & Log::operator << (int i) {
-    fprintf (stderr, "%d", i);
+    FILE *out = getLog ();
+    if (out)
+        fprintf (out, "%d", i);
     return *this;
 }
 
 KMPLAYER_NO_EXPORT Log & Log::operator << (unsigned int i) {
-    fprintf (stderr, "%u", i);
+    FILE *out = getLog ();
+    if (out)
+        fprintf (out, "%u", i);
     return *this;
 }
 
 KMPLAYER_NO_EXPORT Log & Log::operator << (double d) {
-    fprintf (stderr, "%f", d);
+    FILE *out = getLog ();
+    if (out)
+        fprintf (out, "%f", d);
     return *this;
 }
 
 KMPLAYER_NO_EXPORT Log & Log::operator << (void * d) {
-    fprintf (stderr, "0x%x", d);
+    FILE *out = getLog ();
+    if (out)
+        fprintf (out, "0x%x", d);
     return *this;
 }
