@@ -46,6 +46,7 @@ static gchar *start_url;
 
 static void *browser_data;
 static NSNotify *browser_notify;
+static NPNetscapeFuncs ns_funcs;
 static NPPluginFuncs np_funcs;       /* plugin functions              */
 static NPP npp;                      /* single instance of the plugin */
 static NPWindow np_window;
@@ -73,7 +74,6 @@ typedef struct _StreamInfo {
     bool called_plugin;
     bool destroyed;
 } StreamInfo;
-struct JsObject;
 typedef struct _JsObject {
     NPObject npobject;
     char * name;
@@ -162,10 +162,12 @@ static int32_t writeStream (uint32_t stream, const char *buf, uint32_t count) {
             if (!si->called_plugin) {
                 NPError err;
                 uint16 stype = NP_NORMAL;
-                err = np_funcs.newstream (npp, si->mimetype ? si->mimetype : "text/plain", &si->np_stream, 0, &stype);
+                err = np_funcs.newstream (npp,
+                        si->mimetype ? si->mimetype : (char *)"text/plain",
+                        &si->np_stream, 0, &stype);
                 if (err != NPERR_NO_ERROR) {
                     g_printerr ("newstream error %d\n", err);
-                    destroyStream (stream);
+                    destroyStream ((void *)stream);
                     return count;
                 }
                 print ("newStream %d type:%d %s mime:%s\n", (long) stream, stype, si->url, si->mimetype ? si->mimetype : "text/plain");
@@ -175,7 +177,7 @@ static int32_t writeStream (uint32_t stream, const char *buf, uint32_t count) {
                 sz = np_funcs.writeready (npp, &si->np_stream);
             if (sz > 0) {
                 sz = np_funcs.write (npp, &si->np_stream, si->stream_pos,
-                        (int32_t) count > sz ? sz : (int32_t) count, buf);
+                        (int32_t) count > sz ? sz : (int32_t) count, (char *)buf);
                 if (sz < 0) { /*FIXME plugin destroys stream here*/
                     destroyStream ((gpointer)stream);
                     return 0;
@@ -255,7 +257,7 @@ static void nsReleaseObject (NPObject *npobj) {
 
 static NPError nsGetURL (NPP instance, const char* url, const char* target) {
     (void)instance;
-    print ("nsGetURL %s %s\n", url, target);
+    print ("nsGetURL %s %s\n", url, target ? target : "");
     addStream (url, 0L, target, 0, NULL, 0L, false);
     return NPERR_NO_ERROR;
 }
@@ -263,7 +265,7 @@ static NPError nsGetURL (NPP instance, const char* url, const char* target) {
 static NPError nsPostURL (NPP instance, const char *url,
         const char *target, uint32 len, const char *buf, NPBool file) {
     (void)instance; (void)file;
-    print ("nsPostURL %s %s\n", url, target);
+    print ("nsPostURL %s %s\n", url, target ? target : "");
     addStream (url, 0L, target, len, buf, 0L, false);
     return NPERR_NO_ERROR;
 }
@@ -360,13 +362,13 @@ static NPError nsGetValue (NPP instance, NPNVariable variable, void *value) {
             *(int*)value = 0;
             break;
         case NPNVjavascriptEnabledBool:
-            *(int*)value = 1;
+            *(NPBool*)value = 1;
             break;
         case NPNVasdEnabledBool:
-            *(int*)value = 0;
+            *(NPBool*)value = 0;
             break;
         case NPNVisOfflineBool:
-            *(int*)value = 0;
+            *(NPBool*)value = 0;
             break;
         case NPNVserviceManager:
             *(int*)value = 0;
@@ -375,7 +377,7 @@ static NPError nsGetValue (NPP instance, NPNVariable variable, void *value) {
             *(int*)value = NPNVGtk2;
             break;
         case NPNVSupportsXEmbedBool:
-            *(int*)value = 1;
+            *(NPBool*)value = 1;
             break;
         case NPNVWindowNPObject:
             if (!js_window) {
@@ -392,8 +394,8 @@ static NPError nsGetValue (NPP instance, NPNVariable variable, void *value) {
             break;
         }*/
         default:
-            *(int*)value = 0;
             print ("unknown value\n");
+            return NPERR_GENERIC_ERROR;
     }
     return NPERR_NO_ERROR;
 }
@@ -447,7 +449,7 @@ static bool nsIdentifierIsString (NPIdentifier name) {
 }
 
 static NPUTF8 * nsUTF8FromIdentifier (NPIdentifier name) {
-    char *str = g_tree_lookup (identifiers, name);
+    char *str = (char *) g_tree_lookup (identifiers, name);
     print ("NPN_UTF8FromIdentifier\n");
     if (str)
         return (NPUTF8 *)g_strdup (str);
@@ -465,7 +467,7 @@ static bool nsInvoke (NPP instance, NPObject * npobj, NPIdentifier method,
     (void)instance;
     if (!obj) {
         if (!scriptable_peer &&
-            NPERR_NO_ERROR != np_funcs.getvalue ((void*)npp,
+            NPERR_NO_ERROR != np_funcs.getvalue (npp,
                 NPPVpluginScriptableNPObject, (void*)&scriptable_peer))
             return false;
         obj = scriptable_peer;
@@ -554,7 +556,7 @@ static void nsReleaseVariantValue (NPVariant * variant) {
 
 static void nsSetException (NPObject *npobj, const NPUTF8 *message) {
     (void)npobj;
-    print ("NPN_SetException %s\n", message);
+    print ("NPN_SetException %s\n", message ? message : "");
 }
 
 static bool nsPushPopupsEnabledState (NPP instance, NPBool enabled) {
@@ -668,8 +670,8 @@ static bool windowClassGetProperty (NPObject *npobj, NPIdentifier property,
 static bool createStartLink (NPVariant *result) {
     result->type = NPVariantType_String;
     result->value.stringValue.utf8length = strlen (start_url);
-    result->value.stringValue.utf8characters = nsAlloc (strlen (start_url) + 1);
-    strcpy (result->value.stringValue.utf8characters, start_url);
+    result->value.stringValue.utf8characters = (char *) nsAlloc (strlen (start_url) + 1);
+    strcpy ((char *) result->value.stringValue.utf8characters, start_url);
     return true;
 }
 
@@ -705,7 +707,6 @@ void nppPluginShutdown (NPPluginLib *lib) {
 }
 
 NPPluginLib *nppPluginInit (const char *lib) {
-    NPNetscapeFuncs ns_funcs;
     NPError np_err;
     NPPluginLib *plugin_lib;
     GModule *library;
@@ -811,7 +812,7 @@ NPPluginLib *nppPluginInit (const char *lib) {
         goto bail_out;
     }
 
-    identifiers = g_tree_new (strcmp);
+    identifiers = g_tree_new ((GCompareFunc) strcmp);
     stream_list = g_tree_new (streamCompare);
 
     return plugin_lib;
@@ -846,14 +847,14 @@ void *nppInstanceOpen (const char *mime, uint16_t argc, char *argn[],
     if (width > 0 && height > 0)
         notify->setDimension (ndata, width, height);*/
 
-    np_err = np_funcs.newp (mime, npp, NP_EMBED, argc, argn, argv, saved_data);
+    np_err = np_funcs.newp ((char *)mime, npp, NP_EMBED, argc, argn, argv, saved_data);
     if (np_err != NPERR_NO_ERROR) {
         print ("NPP_New failure %d %p %p\n", np_err, np_funcs, np_funcs.newp);
         nsMemFree (npp);
         return NULL;
     }
     if (np_funcs.getvalue) {
-        np_err = np_funcs.getvalue ((void*)npp,
+        np_err = np_funcs.getvalue (npp,
                 NPPVpluginNeedsXEmbed, (void*)&needs_xembed);
         if (np_err != NPERR_NO_ERROR || !needs_xembed) {
             print ("NPP_GetValue NPPVpluginNeedsXEmbed failure %d\n", np_err);
@@ -872,8 +873,8 @@ void *nppInstanceOpen (const char *mime, uint16_t argc, char *argn[],
     np_window.type = NPWindowTypeWindow;
     ws_info.type = NP_SETWINDOW;
     screen = DefaultScreen (display);
-    ws_info.display = (void*)(long)display;
-    ws_info.visual = (void*)(long)DefaultVisual (display, screen);
+    ws_info.display = display;
+    ws_info.visual = DefaultVisual (display, screen);
     ws_info.colormap = DefaultColormap (display, screen);
     ws_info.depth = DefaultDepth (display, screen);
     print ("display %u %dx%d\n", socket_id, np_window.width, np_window.height);
@@ -911,7 +912,7 @@ void nppStreamFinished (uint32_t stream, uint32_t reason, void *pdata) {
 }
 
 void nppStreamRedirected (uint32_t stream, const char *url) {
-    StreamInfo *si = (StreamInfo *) g_tree_lookup (stream_list, stream);
+    StreamInfo *si = (StreamInfo *) g_tree_lookup (stream_list, (void *) stream);
     if (si) {
         g_free (si->url);
         si->url = g_strdup (url);
@@ -920,7 +921,7 @@ void nppStreamRedirected (uint32_t stream, const char *url) {
 }
 
 void nppStreamInfo (uint32_t sid, const char *mime, unsigned long length) {
-    StreamInfo *si = (StreamInfo *) g_tree_lookup (stream_list, sid);
+    StreamInfo *si = (StreamInfo *) g_tree_lookup (stream_list, (void *) sid);
     if (si) {
         print ("nppStreamInfo %s %s\n", si->url, mime ? mime : "");
         if (si->mimetype)
